"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fs = require("fs");
var mkdirp = require("mkdirp");
var path = require("path");
var commandOptions_1 = require("./commandOptions");
var index_1 = require("./index");
var utils_1 = require("./utils");
function readSchemasFromStdin() {
    process.stdin.setEncoding('utf-8');
    return new Promise(function (resolve, reject) {
        var data = '';
        function onRead() {
            var chunk;
            while (chunk = process.stdin.read()) {
                if (typeof chunk === 'string') {
                    data += chunk;
                }
            }
        }
        function onEnd() {
            var schemas = utils_1.parseFileContent(data);
            if (!Array.isArray(schemas)) {
                schemas = [schemas];
            }
            resolve(schemas);
        }
        function onError(err) {
            reject(err);
        }
        process.stdin
            .on('readable', onRead)
            .once('end', onEnd)
            .once('error', onError);
    });
}
function exec() {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var schemas;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    commandOptions_1.initialize(process.argv);
                    schemas = [];
                    if (!commandOptions_1.default.isReadFromStdin()) return [3, 2];
                    return [4, readSchemasFromStdin()];
                case 1:
                    schemas = _a.sent();
                    _a.label = 2;
                case 2:
                    index_1.default(schemas).then(function (result) {
                        if (commandOptions_1.default.out) {
                            mkdirp.sync(path.dirname(commandOptions_1.default.out));
                            fs.writeFileSync(commandOptions_1.default.out, result, { encoding: 'utf-8' });
                        }
                        else {
                            console.log(result);
                        }
                    }).catch(function (err) {
                        console.error(err.stack || err);
                    });
                    return [2];
            }
        });
    });
}
exec();
