"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Debug = require("debug");
var fs = require("fs");
var glob = require("glob");
var request = require("request");
var commandOptions_1 = require("./commandOptions");
var JsonPointer = require("./jsonPointer");
var schemaid_1 = require("./schemaid");
var typeDefinition_1 = require("./typeDefinition");
var utils_1 = require("./utils");
var writeProcessor_1 = require("./writeProcessor");
var debug = Debug('dtsgen');
var walkMaker = '<<type>>';
var JsonSchemaParser = (function () {
    function JsonSchemaParser() {
        this.typeCache = new Map();
        this.schemaReference = new Map();
        this.referenceCache = new Map();
    }
    JsonSchemaParser.prototype.generateDts = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var _a, _b, typeId, _c, _d, ref, _e, _f, _g, schema, map, _h, _j, id, process, env, e_1, _k, e_2, _l, e_3, _m, e_4, _o;
            return tslib_1.__generator(this, function (_p) {
                switch (_p.label) {
                    case 0:
                        debug("generate d.ts.");
                        return [4, this.resolveReference()];
                    case 1:
                        _p.sent();
                        if (debug.enabled) {
                            debug('TypeId list:');
                            try {
                                for (_a = tslib_1.__values(this.typeCache.keys()), _b = _a.next(); !_b.done; _b = _a.next()) {
                                    typeId = _b.value;
                                    debug('  ' + typeId);
                                }
                            }
                            catch (e_1_1) { e_1 = { error: e_1_1 }; }
                            finally {
                                try {
                                    if (_b && !_b.done && (_k = _a.return)) _k.call(_a);
                                }
                                finally { if (e_1) throw e_1.error; }
                            }
                            debug('SchemaId list:');
                            try {
                                for (_c = tslib_1.__values(this.schemaReference.keys()), _d = _c.next(); !_d.done; _d = _c.next()) {
                                    ref = _d.value;
                                    debug('  ' + ref);
                                }
                            }
                            catch (e_2_1) { e_2 = { error: e_2_1 }; }
                            finally {
                                try {
                                    if (_d && !_d.done && (_l = _c.return)) _l.call(_c);
                                }
                                finally { if (e_2) throw e_2.error; }
                            }
                            debug('Reference list:');
                            try {
                                for (_e = tslib_1.__values(this.referenceCache.entries()), _f = _e.next(); !_f.done; _f = _e.next()) {
                                    _g = tslib_1.__read(_f.value, 2), schema = _g[0], map = _g[1];
                                    debug('  ' + schema.id);
                                    try {
                                        for (_h = tslib_1.__values(map.keys()), _j = _h.next(); !_j.done; _j = _h.next()) {
                                            id = _j.value;
                                            debug('    ' + id);
                                        }
                                    }
                                    catch (e_4_1) { e_4 = { error: e_4_1 }; }
                                    finally {
                                        try {
                                            if (_j && !_j.done && (_o = _h.return)) _o.call(_h);
                                        }
                                        finally { if (e_4) throw e_4.error; }
                                    }
                                }
                            }
                            catch (e_3_1) { e_3 = { error: e_3_1 }; }
                            finally {
                                try {
                                    if (_f && !_f.done && (_m = _e.return)) _m.call(_e);
                                }
                                finally { if (e_3) throw e_3.error; }
                            }
                        }
                        process = new writeProcessor_1.WriteProcessor(function (baseSchema, ref) {
                            debug("Search Reference: schemaId=" + (baseSchema ? baseSchema.id : null) + ", ref=" + ref);
                            var map = _this.referenceCache.get(baseSchema);
                            if (map == null) {
                                return undefined;
                            }
                            var refId = new schemaid_1.SchemaId(ref);
                            var result = map.get(refId.getAbsoluteId());
                            if (result == null) {
                                if (refId.existsJsonPointerHash()) {
                                    var fileId_1 = refId.getFileId();
                                    var schema = (function () {
                                        if (fileId_1 == null) {
                                            return baseSchema;
                                        }
                                        var refSchema = _this.schemaReference.get(fileId_1);
                                        if (refSchema == null) {
                                            throw Error("There is no reference target schema. fileId=" + fileId_1);
                                        }
                                        return refSchema.targetSchema;
                                    })();
                                    debug("  fileId=" + fileId_1 + ", schemaId=" + schema.id + ".");
                                    return JsonPointer.get(schema, refId.getJsonPointerHash());
                                }
                            }
                            return result;
                        });
                        env = this.createHierarchicalMap(this.typeCache);
                        if (commandOptions_1.default.header) {
                            process.outputLine(commandOptions_1.default.header);
                        }
                        this.walk(process, env);
                        return [2, process.toDefinition()];
                }
            });
        });
    };
    JsonSchemaParser.prototype.createHierarchicalMap = function (types) {
        var map = {};
        if (types.size === 0) {
            throw new Error('There is no id in the input schema(s)');
        }
        try {
            for (var _a = tslib_1.__values(types.values()), _b = _a.next(); !_b.done; _b = _a.next()) {
                var type = _b.value;
                var names = type.schemaId.getTypeNames();
                JsonPointer.set(map, names.concat(walkMaker), type);
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
            }
            finally { if (e_5) throw e_5.error; }
        }
        return map;
        var e_5, _c;
    };
    JsonSchemaParser.prototype.walk = function (process, env, path) {
        var _this = this;
        if (path === void 0) { path = []; }
        var keys = Object.keys(env).sort();
        keys.forEach(function (key) {
            var val = env[key];
            var type = val[walkMaker];
            if (type instanceof typeDefinition_1.TypeDefinition) {
                debug("  walk doProcess: path=" + JSON.stringify(path) + ", schemaId=" + type.schemaId.getAbsoluteId());
                type.doProcess(process);
            }
            delete val[walkMaker];
            if (Object.keys(val).length > 0) {
                var nextPath = path.concat(key);
                if (process.indentLevel === 0) {
                    process.output('declare ');
                }
                process.output('namespace ').outputType(key, true).outputLine(' {');
                process.increaseIndent();
                _this.walk(process, val, nextPath);
                process.decreaseIndent();
                process.outputLine('}');
            }
        });
    };
    JsonSchemaParser.prototype.resolveReference = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var error, _a, _b, _c, schema, map, _d, _e, _f, ref, type, refId, fileId, refSchema, fetchedSchema, e_6, pointer, targetSchema, typeDef, target, e_7_1, e_8_1, e_8, _g, e_7, _h;
            return tslib_1.__generator(this, function (_j) {
                switch (_j.label) {
                    case 0:
                        debug("resolve reference: reference schema count=" + this.referenceCache.size + ".");
                        error = [];
                        _j.label = 1;
                    case 1:
                        _j.trys.push([1, 15, 16, 17]);
                        _a = tslib_1.__values(this.referenceCache.entries()), _b = _a.next();
                        _j.label = 2;
                    case 2:
                        if (!!_b.done) return [3, 14];
                        _c = tslib_1.__read(_b.value, 2), schema = _c[0], map = _c[1];
                        _j.label = 3;
                    case 3:
                        _j.trys.push([3, 11, 12, 13]);
                        _d = tslib_1.__values(map.entries()), _e = _d.next();
                        _j.label = 4;
                    case 4:
                        if (!!_e.done) return [3, 10];
                        _f = tslib_1.__read(_e.value, 2), ref = _f[0], type = _f[1];
                        if (type != null) {
                            return [3, 9];
                        }
                        refId = new schemaid_1.SchemaId(ref);
                        fileId = refId.getFileId();
                        refSchema = fileId ? this.schemaReference.get(fileId) : undefined;
                        if (!(fileId && refSchema == null)) return [3, 8];
                        if (!refId.isFetchable()) {
                            error.push("$ref target is not found: " + ref);
                            return [3, 9];
                        }
                        _j.label = 5;
                    case 5:
                        _j.trys.push([5, 7, , 8]);
                        debug("fetch remote schema: id=[" + fileId + "].");
                        return [4, this.fetchRemoteSchema(fileId)];
                    case 6:
                        fetchedSchema = _j.sent();
                        this.parseSchema(fetchedSchema, fileId);
                        return [3, 8];
                    case 7:
                        e_6 = _j.sent();
                        error.push("fail to fetch the $ref target: " + ref + ", " + e_6);
                        return [3, 9];
                    case 8:
                        debug("resolve reference: ref=[" + ref + "]");
                        if (refId.existsJsonPointerHash()) {
                            pointer = refId.getJsonPointerHash();
                            targetSchema = refSchema ? refSchema.rootSchema : schema;
                            typeDef = new typeDefinition_1.TypeDefinition(targetSchema, pointer, refId);
                            map.set(ref, typeDef);
                            this.addType(typeDef);
                        }
                        else {
                            target = this.typeCache.get(ref);
                            if (target == null) {
                                error.push("$ref target is not found: " + ref);
                                return [3, 9];
                            }
                            map.set(ref, target);
                        }
                        _j.label = 9;
                    case 9:
                        _e = _d.next();
                        return [3, 4];
                    case 10: return [3, 13];
                    case 11:
                        e_7_1 = _j.sent();
                        e_7 = { error: e_7_1 };
                        return [3, 13];
                    case 12:
                        try {
                            if (_e && !_e.done && (_h = _d.return)) _h.call(_d);
                        }
                        finally { if (e_7) throw e_7.error; }
                        return [7];
                    case 13:
                        _b = _a.next();
                        return [3, 2];
                    case 14: return [3, 17];
                    case 15:
                        e_8_1 = _j.sent();
                        e_8 = { error: e_8_1 };
                        return [3, 17];
                    case 16:
                        try {
                            if (_b && !_b.done && (_g = _a.return)) _g.call(_a);
                        }
                        finally { if (e_8) throw e_8.error; }
                        return [7];
                    case 17:
                        if (error.length > 0) {
                            throw new Error(error.join('\n'));
                        }
                        return [2, true];
                }
            });
        });
    };
    JsonSchemaParser.prototype.fetchLocalFileSchemas = function (globPath) {
        var files = glob.sync(globPath);
        return Promise.all(files.map(function (file) {
            return new Promise(function (resolve, reject) {
                fs.readFile(file, { encoding: 'utf-8' }, function (err, content) {
                    if (err) {
                        reject(err);
                    }
                    else {
                        try {
                            resolve(utils_1.parseFileContent(content, file));
                        }
                        catch (e) {
                            reject(e);
                        }
                    }
                });
            });
        }));
    };
    JsonSchemaParser.prototype.fetchRemoteSchema = function (url) {
        return new Promise(function (resolve, reject) {
            request.get(url, function (err, response, body) {
                if (err) {
                    return reject(err);
                }
                else if (response.statusCode !== 200) {
                    return reject(body);
                }
                else {
                    try {
                        resolve(utils_1.parseFileContent(body, url));
                    }
                    catch (e) {
                        reject(e);
                    }
                }
            });
        });
    };
    JsonSchemaParser.prototype.parseSchema = function (schema, url) {
        var _this = this;
        if (typeof schema === 'string') {
            schema = JSON.parse(schema);
        }
        debug("parse schema: schemaId=[" + schema.id + "], url=[" + url + "].");
        if (schema.id == null) {
            schema.id = url;
        }
        var walk = function (s, paths) {
            function walkArray(array, pathArray) {
                array.forEach(function (item, index) {
                    walk(item, pathArray.concat(index.toString()));
                });
            }
            function walkObject(obj, pathObject, isDefinitions) {
                if (isDefinitions === void 0) { isDefinitions = false; }
                Object.keys(obj).forEach(function (key) {
                    var sub = obj[key];
                    if (sub != null) {
                        if (isDefinitions && sub.id == null) {
                            sub.id = "#/definitions/" + key;
                        }
                        walk(sub, pathObject.concat(key));
                    }
                });
            }
            if (s == null || typeof s !== 'object') {
                return;
            }
            var allOf = s.allOf;
            if (allOf != null) {
                walkArray(allOf, paths.concat('allOf'));
            }
            var anyOf = s.anyOf;
            if (anyOf != null) {
                walkArray(anyOf, paths.concat('anyOf'));
            }
            var oneOf = s.oneOf;
            if (oneOf != null) {
                walkArray(oneOf, paths.concat('oneOf'));
            }
            var items = s.items;
            if (items != null) {
                if (Array.isArray(items)) {
                    walkArray(items, paths.concat('items'));
                }
                else {
                    walk(items, paths.concat('items'));
                }
            }
            var additionalItems = s.additionalItems;
            if (additionalItems != null && typeof additionalItems !== 'boolean') {
                walk(additionalItems, paths.concat('additionalItems'));
            }
            var definitions = s.definitions;
            if (definitions != null) {
                walkObject(definitions, paths.concat('definitions'), true);
            }
            var properties = s.properties;
            if (properties != null) {
                walkObject(properties, paths.concat('properties'));
            }
            var patternProperties = s.patternProperties;
            if (patternProperties != null) {
                walkObject(patternProperties, paths.concat('patternProperties'));
            }
            var additionalProperties = s.additionalProperties;
            if (additionalProperties != null && typeof additionalProperties !== 'boolean') {
                walk(additionalProperties, paths.concat('additionalProperties'));
            }
            if (typeof s.id === 'string') {
                var type = new typeDefinition_1.TypeDefinition(schema, paths);
                s.id = type.schemaId.getAbsoluteId();
                _this.addType(type);
            }
            if (typeof s.$ref === 'string') {
                s.$ref = _this.addReference(schema, s.$ref);
            }
        };
        walk(schema, []);
    };
    JsonSchemaParser.prototype.addType = function (g) {
        var id = g.schemaId;
        if (id) {
            this.typeCache.set(id.getAbsoluteId(), g);
            debug("add type: id=" + id.getAbsoluteId());
            var fileId = id.getFileId();
            if (!this.schemaReference.has(fileId)) {
                this.schemaReference.set(fileId, g);
            }
        }
    };
    JsonSchemaParser.prototype.addReference = function (schema, ref) {
        var map = this.referenceCache.get(schema);
        if (map == null) {
            map = new Map();
            this.referenceCache.set(schema, map);
        }
        var refId = new schemaid_1.SchemaId(ref, schema.id ? [schema.id] : undefined);
        map.set(refId.getAbsoluteId(), undefined);
        return refId.getAbsoluteId();
    };
    JsonSchemaParser.prototype.clear = function () {
        debug('clear data cache.');
        this.typeCache.clear();
        this.schemaReference.clear();
        this.referenceCache.clear();
    };
    return JsonSchemaParser;
}());
exports.JsonSchemaParser = JsonSchemaParser;
